#!/usr/bin/perl

use strict;
use Test::More tests => 44;
use FDN::Common::Logger;
use FDN::Common::Constantes;
use FDN::Common::Config;
use FDN::Sql::DB_Access;
use FDN::Sql::Basic;
use FDN::Sql::DBC;


# 1
BEGIN { use_ok('FDN::Sql::Plutar::Basic') }

# 2
BEGIN { use_ok('FDN::Sql::Plutar::Constantes') }

# 3
BEGIN { use_ok('FDN::Sql::Plutar::Run') }

# 4,5
$FDN::Common::Config::CONFIG_ROOT = './t/';
my $dbh = do_connect('test');
ok ( defined $dbh, 'do_connect' );
is ( ref($dbh), 'HASH', 'do_connect' );

# 6
ok ( -f './sql/plutar.sql', 'plutar.sql' );
open FILE, '<./sql/plutar.sql';
my $plutar;
{ local $/;
  $plutar = <FILE>; }
close FILE;
my @queries = split /;\s*$/m, $plutar;
my $tmp;
my $res = TRUE;
for my $q ( @queries ) {
	next if ( $q =~ /drop table if exists SEQ/ );
	next if ( $q =~ /create table SEQ/ );
	$tmp = sql_transaction($dbh,'create',$q);
	if ( ! $tmp->{'ok'} ) {
		$res = FALSE;
		print "# Failed query: $q\n";
	}
}
# 7
ok ( $res, 'plutar.sql' );

# 8
$res = plutar_select_db('test');
ok ( $res, 'plutar_select_db' );

# 9
$res = plutar_select_config('plutar.cfg');
ok ( $res, 'plutar_select_config');

# 10
my $args = {
	'clef' => 'val',
	'autre' => 'truc',
	'return' => 'success'
};
$res = new_plutar('essai','now');
is ( $res, 1, 'plutar_task' );
set_plutar_properties(1,$args);

# 11
$res  = plutar_run_task($args,'test3');
is ( $res, PLUTAR_SUCCESS, 'plutar_run_task' );

# 12
$res  = plutar_run_task($args,'test1' );
is ( $res, PLUTAR_FAILED,  'plutar_run_task' );

# 13
$res  = plutar_run_task($args,'test2'  );
is ( $res, PLUTAR_RETRY,   'plutar_run_task' );

# 14
$res  = plutar_run_task($args,'test');
is ( $res, PLUTAR_SUCCESS, 'plutar_run_task' );

# 15
# Ending the previous task
$res = plutar_cancel_task(1);
ok ( $res, 'plutar_cancel_task' );

# New task, that will be run in two steps
# 16
$res  = new_plutar('test','now');
is ( $res, 2, 'new_plutar' );

$args = {
	'clef' => 'val',
	'autre' => 'truc',
	'return' => 'retry'
};
set_plutar_properties(2,$args);

# First step, change 'return' from 'retry' to 'success'
# 17,22
my $task;
my $pid;
my $act;
$task = first_plutar();
$pid  = $task->{'PLUTAR_ID'};
$act  = $task->{'PLUTAR_ACTION'};
is ( $pid, 2, 'first_plutar');
$args = get_plutar_properties($pid);
is ( ref($args), 'HASH', 'plutar_get_args' );
is ( $args->{'return'}, 'retry', 'plutar_get_args' );

$res = plutar_runing_task($pid,1);
ok ( $res, 'plutar_runing_task' );

$res  = plutar_run_task($args,$act);
is ( $res, PLUTAR_RETRY, 'plutar_run_task');
update_plutar_properties($pid,$args);

$res  = plutar_after_run($pid,PLUTAR_RETRY,$act);
is ( $res, 'Execution of plutar task 2 retry within 0 seconds.', 'plutar_retry_task' );

# Second step, does success
# 23,28
$task = first_plutar();
$pid  = $task->{'PLUTAR_ID'};
$act  = $task->{'PLUTAR_ACTION'};
is ( $pid, 2, 'first_plutar');
$args = get_plutar_properties($pid);
is ( ref($args), 'HASH', 'get_plutar_properties' );
is ( $args->{'return'}, 'success', 'get_plutar_properties' );

$res = plutar_runing_task($pid,2);
ok ( $res, 'plutar_runing_task' );

$res  = plutar_run_task($args,$act);
is ( $res, PLUTAR_SUCCESS, 'plutar_run_task');
update_plutar_properties($pid,$args);

$res = plutar_after_run($pid,PLUTAR_SUCCESS,$act);
is ( $res, 'Execution of plutar task 2 ok', 'plutar_after_run' );

# New task, that will fail on the first run.
# 29
$res  = new_plutar('test','now');
is ( $res, 3, 'new_plutar' );

$args = {
	'return' => 'fail'
};
set_plutar_properties(3,$args);


# First (and only) run
# 30,33
$task = first_plutar();
$pid  = $task->{'PLUTAR_ID'};
$act  = $task->{'PLUTAR_ACTION'};
is ( $pid, 3, 'first_plutar');
$args = get_plutar_properties($pid);
$res = plutar_runing_task($pid,1);
ok ( $res, 'plutar_runing_task' );
$res  = plutar_run_task($args,$act);
is ( $res, PLUTAR_FAILED, 'plutar_run_task');
$res = plutar_after_run($pid,PLUTAR_FAILED,$act);
is ( $res, 'Execution of plutar task 3 failed', 'plutar_fail_task');

# New task, that will return retry and thus fail because there is
# no more run.
# 34
$res  = new_plutar('test4','now');
is ( $res, 4, 'new_plutar' );

$args = {
	'clef' => 'val',
	'autre' => 'truc',
	'return' => 'retry'
};
set_plutar_properties(4,$args);

# First step, change 'return' from 'retry' to 'success'
# 35,40
$task = first_plutar();
$pid  = $task->{'PLUTAR_ID'};
$act  = $task->{'PLUTAR_ACTION'};
is ( $pid, 4, 'first_plutar');
$args = get_plutar_properties($pid);
is ( ref($args), 'HASH', 'get_plutar_properties' );
is ( $args->{'return'}, 'retry', 'get_plutar_properties' );

$res = plutar_runing_task($pid,1);
ok ( $res, 'plutar_runing_task' );

$res  = plutar_run_task($args,$act);
is ( $res, PLUTAR_RETRY, 'plutar_run_task');
update_plutar_properties($pid,$args);

$res  = plutar_after_run($pid,PLUTAR_RETRY,$act);
is ( $res, 'No more trial for plutar task 4 (already 1), thus task failed.', 'plutar_retry_task' );

# 41
$res = plutar_etat_par_id(1);
is ( $res, 'canceled', 'plutar_etat_par_id');

# 42
$res = plutar_etat_par_id(2);
is ( $res, 'done', 'plutar_etat_par_id');

# 43
$res = plutar_etat_par_id(3);
is ( $res, 'failed', 'plutar_etat_par_id');

# 44
$res = plutar_etat_par_id(4);
is ( $res, 'failed', 'plutar_etat_par_id');

